
// instantiate
Word2Vec model;
// pre-trained model to load
me.dir() + "glove-wiki-gigaword-50-tsne-2.txt" => string filepath;
// load pre-trained model (see URLs above for download)
if( !model.load( filepath ) )
{
    <<< "cannot load model:", filepath >>>;
    me.exit();
}

// my future
["good", "healthy", "traumatized", "loved", "inspiring",
"depressed", "dying", "withered", "joyous", "stressed",
"unloved", "human" ] @=> string myfuture[];

// your future
["static", "same", "lame", "artistic", "overpowering",
"omniscient", "sentient", "utilitarian", "accurate", "humbling" ] @=> string yourfuture[];

// starter "i feel" emotion words with other connotations
["destroyed", "calamitous", "disastrous", "worse", "disbelieving", "apocyptic",
"complicated", "surviving", "hopeful", "helpless", "resistant" ] @=> string others[];

// conditions
4 => int LINES_PER_STANZA;
3 => int NUM_SECTIONS; // here each section is a fixed stanzas format

// number of nearest words to retrieve for each word
// higher this number the higher the variance per word
66 => int K_NEAREST;

// timing
420::ms => dur T_WORD; // duration per word
false => int shouldScaleTimeToWordLength; // longer words take longer?
T_WORD => dur T_LINE_PAUSE; // a little pause after each line
T_WORD * 2 => dur T_STANZA_PAUSE; // pause after each stanza

// sound
SinOsc crystal => ADSR adsr1 => Gain left => dac.left;
crystal => ADSR adsr2 => Gain right => dac.right;
// feedback delay for echos
left => DelayL DL => left; .5 => DL.gain; T_WORD*1.5 => DL.max => DL.delay;
right => DelayL DR => right; .3 => DR.gain; T_WORD*2 => DR.max => DR.delay;

0.1 => left.gain => right.gain;
adsr1.set(0.1::second, 0.1::second, 0.7, 0.1::second);
adsr2.set(0.1::second, 0.1::second, 0.7, 0.1::second);
adsr1.keyOff(); adsr2.keyOff();

// current word
string prediction;
// word vector
float vec[model.dim()];
// search results
string words[K_NEAREST];

// line break
chout <= IO.newline(); chout.flush();

poemWrite();

// loop over stanzas
fun void poemWrite()
{
    adsr1.keyOn(); adsr2.keyOn();
    startStanza1();
    adsr1.keyOff(); adsr2.keyOff();
    // grab a word out of the "good" bag
    myfuture[Math.random2(0,myfuture.size()-1)] => prediction;
    // print a stanza
    wait(1::second);
    stanza1(prediction, LINES_PER_STANZA);
    
    adsr1.keyOn(); adsr2.keyOn();
    startStanza2();
    adsr1.keyOff(); adsr2.keyOff();
    // grab a word out of the "bad" bag
    yourfuture[Math.random2(0,yourfuture.size()-1)] => prediction;
    // print a stanza
    wait(1::second);
    stanza2( prediction, LINES_PER_STANZA, 43, 58 );
    
    adsr1.keyOn(); adsr2.keyOn();
    startStanza3();
    // grab a word out of the "other" bag
    others[Math.random2(0,others.size()-1)] => prediction;
    // print a stanza
    wait(1::second);
    stanza3(prediction, 666, Math.random2(20,94)*2 );
    
    chout <= IO.newline() <= IO.newline(); chout.flush();
    wait(1::second);
    endStanza();
    chout <= IO.newline() <= IO.newline(); chout.flush();
    // pause at end of line
    T_STANZA_PAUSE => now;

}

// print at the end
chout <= "\"Future\"" <= IO.newline();
chout <= "-- A prophetic pseudo-sonnet." <= IO.newline();
chout.flush();

// print a stanza from a starter prediction word
fun void stanza1(string prediction, int numLines)
{
    
    // loop over lines in a stanza
    for( int n; n < numLines; n++ )
    {
        spork ~ willBe("You");
        bellRing();
    }
    // next line!
    chout <= IO.newline(); chout.flush();
    // pause at end of line
    T_LINE_PAUSE => now;
}

fun void willBe(string subject)
{

    say(subject); wait();
    say("will"); wait();
    say("be"); wait();
    say(prediction); wait();
    endl(); wait();
    
    // get similar words
    model.getSimilar( prediction, words.size(), words );
    // choose one at random
    words[Math.random2(0,words.size()-1)] => prediction;
}

fun void stanza2(string prediction, int numLines, int pitch1, int pitch2 )
{
    // loop over lines in a stanza
    for( int n; n < numLines; n++ )
    {
        spork ~ willBe("I");
        bellRing();
    }
    // next line!
    chout <= IO.newline(); chout.flush();
    // pause at end of line
    T_LINE_PAUSE => now;
}

fun void stanza3(string prediction, int numLines, int pitch1)
{
    say("Humanity"); play(pitch1); wait();
    say("will"); wait();
    say("be"); wait();
    say("..."); wait();
    // next line!
    chout <= IO.newline(); chout.flush();
    
    0.4::second => dur waitTime;
    
    SinOsc morningStar => ADSR adsr3 => NRev n2 => dac;
    0.2 => n2.mix; 0.05 => n2.gain;
    adsr3.set(0.05::second, 0.05::second, 0.5, 0.05::second);
    6666.6 => morningStar.freq;
    6666.6 => float curFreq;
    adsr3.keyOn();
    // loop over lines in a stanza
    for( int n; n < numLines; n++ )
    {
        say(prediction); play(Math.random2((20, 94))); wait(waitTime);
        
        // get similar words
        model.getSimilar( prediction, words.size(), words );
        // choose one at random
        words[Math.random2(0,words.size()-1)] => prediction;
        waitTime * 0.98 => waitTime;
        0.98 *=> curFreq;
        curFreq => morningStar.freq;
    }
    // next line!
    chout <= IO.newline(); chout.flush();
    // pause at end of line
    adsr3.keyOff();
    T_LINE_PAUSE => now;
}

fun void startStanza1()
{
    // stanza break; no new sound for now
    say("ChAI,"); wait(0.6::second);
    say("tell"); wait();
    say("me"); wait();
    say("about"); wait();
    say("my"); wait();
    say("future."); wait();

    // next line!
    chout <= IO.newline(); chout.flush();
    chout <= IO.newline(); chout.flush();
    // pause at end of line
    T_LINE_PAUSE => now;
}

fun void startStanza2()
{
    // stanza break; no new sound for now
    say("ChAI,"); wait(0.6::second);
    say("tell"); wait();
    say("me"); wait();
    say("about"); wait();
    say("your"); wait();
    say("future."); wait();

    // next line!
    chout <= IO.newline(); chout.flush();
    chout <= IO.newline(); chout.flush();
    // pause at end of line
    T_LINE_PAUSE => now;
}


fun void startStanza3()
{
    // stanza break; no new sound for now
    say("ChAI,"); wait(0.6::second);
    say("tell"); wait();
    say("me"); wait();
    say("about"); wait();
    say("our"); wait();
    say("future."); wait();

    // next line!
    chout <= IO.newline(); chout.flush();
    chout <= IO.newline(); chout.flush();
    // pause at end of line
    T_LINE_PAUSE => now;
}

fun void endStanza()
{
    0.05 => left.gain => right.gain;
    chout <= IO.newline(); chout.flush();
    chout <= IO.newline(); chout.flush();
    chout <= IO.newline(); chout.flush();
    // stanza break; no new sound for now
    wait();
    say("ChAI,"); wait(0.6::second);
    say("what"); wait();
    say("do"); wait();
    say("you"); wait();
    say("know?"); wait(1.5::second);

    // next line!
    chout <= IO.newline(); chout.flush();
    chout <= IO.newline(); chout.flush();
    say("Everything"); wait(0.7::second);
    say("or"); wait();
    say("nothing,"); wait();
    say("depending"); wait();
    say("on"); wait();
    say("how"); wait();
    say("you"); wait();
    say("see"); wait();
    say("it."); wait(1.5::second);

    chout <= IO.newline(); chout.flush();
    chout <= IO.newline(); chout.flush();
    say("ChAI,"); wait(0.7::second);
    say("how"); wait();
    say("do"); wait();
    say("you"); wait();
    say("feel?"); wait(2::second);

    chout <= IO.newline(); chout.flush();
    chout <= IO.newline(); chout.flush();
    dopeEnding();
    // pause at end of line
    adsr1.keyOff(); adsr2.keyOff();
    T_LINE_PAUSE => now;
}

fun void dopeEnding()
{
    say("DOPE!"); 
    SndBuf dope => NRev r => ResonZ rz => dac;
    "special:dope" => dope.read;
    wait(2::second);
}

// say a word with space after
fun void say( string word )
{
    say( word, " " );    
}

// say a word
fun void say( string word, string append )
{
    // print it
    chout <= word <= append; chout.flush();
}

// sonify
fun void play( int pitch )
{
    play( pitch, Math.random2f(.8,1) );
}

// sonify
fun void play( int pitch, float velocity )
{
    // convert pitch to frequency and set it
    pitch => Std.mtof => crystal.freq;
}

// wait
fun void wait()
{
    wait( T_WORD );
}

// wait
fun void wait( dur T )
{    
    // let time pass, let sound...sound
    T => now;
}

// new line with timing
fun void endl()
{
    endl( T_WORD );
}

// new line with timing
fun void endl( dur T )
{
    // new line
    chout <= IO.newline(); chout.flush();
    // let time pass
    T => now;
}

fun void bellRing()
{
    KrstlChr tb => ADSR adsr => NRev n1 => Echo a => Echo b => ResonZ r => dac;

    Std.mtof(44) => tb.freq;
    0.2 => n1.mix;

    adsr.set(0.05::second, 0.5::second, 0.5, 1.5::second);
    adsr.keyOn();
    0.9 => tb.noteOn;

    1::second => now;
    0.7 => tb.noteOn;
    adsr.keyOff();
    adsr.releaseTime() => now;
    0.5::second => now;
}
